import re
import os


def delete_files_except_best(run_id = None, epochs = None, path = None):
    directory = path['save_model_path']

    pattern1 = run_id
    pattern2 = run_id + '-epoch-' + epochs.zfill(5)
    regex1 = re.compile(pattern1)
    regex2 = re.compile(pattern2)
    
    for subdir, dirs, files in os.walk(directory):
        for filename in files:
            if regex1.search(filename) and not regex2.search(filename):
                os.remove(os.path.join(directory, filename))


def get_best_model_path(run_id = None, path = None):
    hdf5_path = path['save_model_path']
    pattern = str(run_id) 
    regex = re.compile(pattern)
    best_model_path = []
    best_model_path_dict = dict()
    for subdir, dirs, files in os.walk(hdf5_path):
        for filename in files:
            if regex.search(filename):
                best_model_path.append(str('/' + filename))

    if 'json' in best_model_path[0]:
        best_model_path_dict['json'] = best_model_path[0]
        best_model_path_dict['h5'] = best_model_path[1]
    else:
        best_model_path_dict['json'] = best_model_path[1]
        best_model_path_dict['h5'] = best_model_path[0]

    return best_model_path_dict




from Accuinsight.modeler.core.LcConst.LcConst import RUN_OBJ_NAME, RUN_OBJ_JSON_PATH, RUN_OBJ_SHAP_PATH, \
    RUN_OBJ_CSV_PATH, SOURCE_FILE_PATH, SOURCE_FILE_NAME, SOURCE_FILE_DIGEST, SOURCE_FILE_GIT_REPO,\
    SOURCE_FILE_GIT_COMMIT_ID, SOURCE_FILE_IS_DIRTY, PYTHON_DEPENDENCY


class RunObject:
    def __init__(self, run_obj={}):
        self.run_obj = run_obj

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        return setattr(self, key, value)

    def set_json_path(self, json_path):
        self.run_obj[RUN_OBJ_JSON_PATH] = json_path

    def set_csv_path(self, csv_path):
        self.run_obj[RUN_OBJ_CSV_PATH] = csv_path

    def set_shap_path(self, shap_path):
        self.run_obj[RUN_OBJ_SHAP_PATH] = shap_path

    def set_run_name(self, run_name):
        self.run_obj[RUN_OBJ_NAME] = run_name

    def set_run_file_path(self, run_file_path):
        self.run_obj[SOURCE_FILE_PATH] = run_file_path

    def set_run_file_name(self, run_file_name):
        self.run_obj[SOURCE_FILE_NAME] = run_file_name

    def set_run_file_md5_hash(self, digest):
        self.run_obj[SOURCE_FILE_DIGEST] = digest

    def set_run_file_git_repo(self, git_url):
        self.run_obj[SOURCE_FILE_GIT_REPO] = git_url

    def set_run_file_git_commit_id(self, git_commit_id):
        self.run_obj[SOURCE_FILE_GIT_COMMIT_ID] = git_commit_id

    def set_run_file_git_is_dirty(self, isdirty):
        self.run_obj[SOURCE_FILE_IS_DIRTY] = isdirty

    def set_run_file_python_dependency(self, dependency):
        self.run_obj[PYTHON_DEPENDENCY] = dependency



import inspect
import json
import warnings
from collections import OrderedDict
import gorilla
import logging
import numpy as np
import keras
from keras.callbacks import CSVLogger
from Accuinsight.modeler.core import func, path, get
from Accuinsight.modeler.core.func import get_time
from Accuinsight.modeler.core.LcConst import LcConst
from Accuinsight.modeler.core.LcConst.LcConst import RUN_NAME_KERAS
from Accuinsight.modeler.core.get_for_visual import roc_pr_curve, get_visual_info_regressor
from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear_runs, set_model_json_path, \
    set_visual_csv_path, set_visual_json_path, set_best_model_h5_path, set_best_model_json_path, \
    set_python_dependencies, set_run_name, set_model_file_path, set_prefix_path, set_shap_json_path
from Accuinsight.modeler.core.feature_contribution import shap_value
from Accuinsight.modeler.utils.dl_utils import delete_files_except_best, get_best_model_path
from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_os_env
from Accuinsight.modeler.clients.modeler_api import LifecycleRestApi
from Accuinsight.Lifecycle.common import Common

logging.basicConfig(level=logging.INFO,
                    format='%(message)s')

warnings.filterwarnings("ignore")


class accuinsight(Common):
    def __init__(self):
        super().__init__()

    def get_file(self, storage_json_file_name=None):
        super().get_file(storage_json_file_name)

    def autolog(self, tag=None, best_weights=False, model_monitor=False):
        global description, endpoint, var_model_file_path, message, thresholds, run_id, alarm, alarm_api, shap_on
        description = tag
        endpoint = self.endpoint
        message = self.message
        thresholds = self.thresholds
        alarm = self.workspace_alarm
        alarm_api = self.workspace_alarm_api
        user_id = self.user_id

        if best_weights:
            best_weights_on = True
        else:
            best_weights_on = False

        if model_monitor:
            shap_on = True
            if self.feature_name is None:
                try:
                    if self.data is not None:
                        feature_name = get.feature_name(self.save_path, self.StorageInfo, self.target_name,
                                                        data=self.data)
                        self.data = None
                    else:
                        feature_name = get.feature_name(self.save_path, self.StorageInfo, self.target_name)

                except:
                    pass      # when user did not use <get_file> or <set_feature> function
            else:
                feature_name = self.feature_name
        else:
            shap_on = False
        run_id = None

        # if in notebook
        if is_in_ipython():
            var_model_file_path = self.notebook_info
            _caller_globals = inspect.stack()[1][0].f_globals
            (
                mainfile,
                sources,
                dependencies
            ) = gather_sources_and_dependencies(
                globs=_caller_globals,
                save_git_info=False
            )
        else:
            _caller_globals = inspect.stack()[1][0].f_globals
            (
                mainfile,
                sources,
                dependencies
            ) = gather_sources_and_dependencies(
                globs=_caller_globals,
                save_git_info=True
            )
            var_model_file_path = mainfile['filename']

        class TrainHistoryCallbacks(keras.callbacks.Callback):
            def __init__(self, verbose=1, mode='auto', period=1):
                super(TrainHistoryCallbacks, self).__init__()
                self.verbose = verbose
                self.period = period
                self.best_epochs = 0
                self.epochs_since_last_save = 0
                self.mode = mode
                self.model_summary = OrderedDict()
                self.run_id = None
                self.model_type = None

            def on_train_begin(self, logs={}):
                logging.info('Using autolog(best_weights={}, model_monitor={}'
                             .format(str(best_weights_on), str(shap_on)))

                global start
                start = get_time.now()
                opt = self.model.optimizer.get_config()
                opt_key = list(opt.keys())[1:]
                opt_result = {k: np.float64(opt[k]) for k in opt_key}

                self.model_summary['data_version'] = endpoint
                self.model_summary['model_description'] = description
                self.model_summary['logging_time'] = get_time.logging_time()
                self.model_summary['run_id'] = func.get_run_id()
                self.model_summary['model_type'] = get.model_type(self.model)

                if user_id is not None:
                    self.model_summary['user_id'] = user_id

                if hasattr(self.model.loss, 'get_config'):
                    self.model_summary['loss_function'] = self.model.loss.get_config()['name']
                else:
                    self.model_summary['loss_function'] = self.model.loss
                self.model_summary['optimizer_info'] = {opt['name']: opt_result}

                '''[get best model] on_train_begin '''
                self.best_weights = self.model.get_weights()

                self.dict_path = path.get_file_path(self.model, usedFramework='keras')

                set_prefix_path(self.dict_path[LcConst.RUN_PREFIX_PATH])

                set_run_name(self.model_summary['model_type'], self.model_summary['run_id'])
                set_python_dependencies(py_depenpency=dependencies)

            '''[get best model] on_epoch_end '''
            def on_epoch_end(self, epoch, logs=None):
                logs = logs or {}

                if epoch == 0:
                    if len(self.model.metrics_names) == 1 and 'loss' in self.model.metrics_names:
                        self.monitor = 'val_loss'
                    elif len(self.model.metrics_names) >= 2:
                        self.monitor = 'val_' + self.model.metrics_names[1]
                    # set monitoring option
                    if self.mode not in ['auto', 'min', 'max']:
                        warnings.warn('GetBest mode %s is unknown, '
                                      'fallback to auto mode.' % (self.mode), RuntimeWarning)
                        self.mode = 'auto'
                    if self.mode == 'min':
                        self.monitor_op = np.less
                        self.best = np.Inf
                    elif self.mode == 'max':
                        self.monitor_op = np.greater
                        self.best = -np.Inf
                    else:
                        if 'acc' in self.monitor or 'f1' in self.monitor:
                            self.monitor_op = np.greater
                            self.best = -np.Inf
                        else:
                            self.monitor_op = np.less
                            self.best = np.Inf
                else:
                    pass
                
                # Using best_weights
                if best_weights_on:
                    # update best_weights
                    self.epochs_since_last_save += 1
                    if self.epochs_since_last_save >= 1:
                        self.epochs_since_last_save = 0
                        current = logs.get(self.monitor)
                        if current is None:
                            warnings.warn('Can pick best model only with %s available, '
                                          'skipping.' % (self.monitor), RuntimeWarning)
                        else:
                            if self.monitor_op(current, self.best):
                                self.best = current
                                self.best_epochs = epoch + 1
                                self.best_weights = self.model.get_weights()
                            else:
                                pass

                    self.current_value = current

                # Not using best_weights
                else:
                    self.last_epoch_metric = logs.get(self.monitor)
                    self.best_epochs = epoch + 1
                    self.current_value = logs.get(self.monitor)

                # model save path
                run_id = self.model_summary['model_type'] + '-' + self.model_summary['run_id']
                common_path = self.dict_path['save_model_path'] + run_id + '-epoch-' + str(epoch + 1).zfill(5) \
                              + '-' + self.monitor + '-' + str(current).zfill(5)
                save_model_path = common_path + '.json'
                save_weights_path = common_path + '.h5'

                # model to JSON
                model_json = self.model.to_json()
                with open(save_model_path, "w") as json_file:
                    json_file.write(model_json)
                    
                # weights to H5
                self.model.save_weights(save_weights_path)
            
            def on_train_end(self, logs={}):
                '''[get best model] on_train_end '''
                if self.verbose > 0:
                    print('Using epoch %05d with %s: %0.5f' % (self.best_epochs, self.monitor, self.best))
                self.model.set_weights(self.best_weights)  # set best model's weights
                
                end = get_time.now()
                self.model_summary['time_delta'] = str(end - start)
                self.model_summary['selected_metrics'] = {self.monitor: self.best}
            
                # path for model_info.json
                set_model_json_path(self.dict_path['model_json'])
                model_json_full_path = self.dict_path['model_json_full']
            
                with open(model_json_full_path, 'w', encoding='utf-8') as save_file:
                    json.dump(self.model_summary, save_file, indent="\t")
            
                if best_weights_on:
                    delete_files_except_best(run_id=self.model_summary['run_id'], epochs=str(self.best_epochs),
                                            path=self.dict_path)
                else:
                    delete_files_except_best(run_id=self.model_summary['run_id'], epochs=str(self.last_epochs),
                    path=self.dict_path)

                path_for_setting_model_json = self.dict_path['save_model_dir'] + \
                                              get_best_model_path(run_id=self.model_summary['run_id'],
                                                                  path=self.dict_path)['json']
                path_for_setting_model_h5 = self.dict_path['save_model_dir'] + \
                                            get_best_model_path(run_id=self.model_summary['run_id'],
                                                                path=self.dict_path)['h5']
                set_best_model_json_path(path_for_setting_model_json)
                set_best_model_h5_path(path_for_setting_model_h5)

                start_ts = int(start.timestamp())
                end_ts = int(end.timestamp())
                delta_ts = end_ts - start_ts

                clear_runs(start_ts, end_ts, delta_ts)
                accuinsight._send_message(metric=self.monitor,
                                          current_value=self.current_value,
                                          message=message,
                                          thresholds=thresholds,
                                          alarm_object=alarm,
                                          alarm_api=alarm_api)
                env_value = get_os_env('ENV')
                modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
                                                env_value[LcConst.BACK_END_API_PORT],
                                                env_value[LcConst.BACK_END_API_URI])
                modeler_rest.lc_create_run()
                accuinsight.off_autolog()

        class visualCallbacks(keras.callbacks.Callback):
            def __init__(self, x_validation=None, y_validation=None):
                super(visualCallbacks, self).__init__()
                self.x_val = x_validation
                self.y_val = y_validation

            def on_train_end(self, logs={}):
                self.dict_path = path.get_file_path(self.model, usedFramework='keras')

                # path for visual.json
                path_for_setting_visual_json = self.dict_path['visual_json']
                visual_json_full_path = self.dict_path['visual_json_full']
                set_visual_json_path(path_for_setting_visual_json)

                # classification
                if get.is_classification(self.model):
                    visual_classification_json = roc_pr_curve(self.model, self.x_val, self.y_val)

                    with open(visual_json_full_path, 'w', encoding='utf-8') as save_file:
                        json.dump(visual_classification_json, save_file, indent="\t")

                # regression
                else:
                    visual_regression_json = OrderedDict()
                    visual_regression_json['True_y'] = self.y_val.tolist()
                    visual_regression_json['Predicted_y'] = get_visual_info_regressor(self.model, self.x_val)

                    with open(visual_json_full_path, 'w', encoding='utf-8') as save_file:
                        json.dump(visual_regression_json, save_file, indent="\t")

        class shapCallbacks(keras.callbacks.Callback):
            def __init__(self, trainX, feature_name, run_id, trigger=shap_on):
                super(shapCallbacks, self).__init__()
                self.trainX = trainX
                self.trigger = trigger
                self.run_id = run_id
                self.feature_name_in_shap = feature_name

            def on_train_end(self, logs={}):
                if self.trigger:
                    self.shap_value = shap_value(self.model, self.trainX, self.feature_name_in_shap)

                    self.dict_path = path.get_file_path(self.model, usedFramework='keras')

                    # path for shap.json
                    shap_json_full_path = self.dict_path['shap_json_full']
                    set_shap_json_path(self.dict_path['shap_json'])

                    with open(shap_json_full_path, 'w', encoding='utf-8') as save_file:
                        json.dump(self.shap_value, save_file, indent='\t')

                else:
                    pass

        def run_and_log_function(self, original, x, y, kwargs, unlogged_params):
            dict_path = path.get_file_path(self, usedFramework='keras')

            path_for_setting_visual_csv = dict_path['visual_csv']
            visual_csv_full_path = dict_path['visual_csv_full']

            # set current run
            set_current_runs(RUN_NAME_KERAS)
            set_model_file_path(var_model_file_path)

            set_visual_csv_path(path_for_setting_visual_csv)

            csv_logger = CSVLogger(visual_csv_full_path, append=True, separator=';')

            # get train data(x) for computing shap value
            if 'x':
                x_train = x
            if shap_on:
                get_shap = shapCallbacks(x_train, feature_name, run_id, trigger=shap_on)
            else:
                pass
 
            ''' save json for visualization '''
            kwargs_dict = OrderedDict()
            
            for key, value in kwargs.items():
                kwargs_dict[key] = value
            
            # using validation_data argument
            if 'validation_data' in kwargs_dict.keys():
                validation_set = kwargs['validation_data']

                try:
                    x_val = validation_set[0]
                    y_val = validation_set[1]

                except:
                    iterator = iter(validation_set)
                    valid_set = next(iterator)
                    x_val = valid_set[0].numpy()
                    y_val = valid_set[1].numpy()
            
#            elif 'validation_split' in kwargs_dict.keys():
#                (x, y), validation_set = (data_adapter.train_validation_split((x,y), validation_split=kwargs_dict['validation_split']))
#                if validation_set:
#                    x_val, y_val, val_sample_weight = (data_adapter.unpack_x_y_sample_weight(validation_set))
#
            else:
                raise ValueError('"validation_data" or "validation_split" does not exist.')
            
            get_visual = visualCallbacks(x_validation=x_val, y_validation=y_val)

            if 'callbacks' in kwargs:
                kwargs['callbacks'] += [csv_logger]
            else:
                kwargs['callbacks'] = [csv_logger]

            kwargs['callbacks'] += [get_visual]
            if shap_on:
                kwargs['callbacks'] += [get_shap]
            else:
                pass
            kwargs['callbacks'] += [TrainHistoryCallbacks()]

            return original(self, x, y, **kwargs)

        @gorilla.patch(keras.Model)
        def fit(self, x, y, **kwargs):
            original = gorilla.get_original_attribute(keras.Model, 'fit')
            unlogged_params = ['self', 'x', 'y', 'callbacks', 'validation_data', 'verbose']
            return run_and_log_function(self, original, x, y, kwargs, unlogged_params)

        settings = gorilla.Settings(allow_hit=True, store_hit=True)
        gorilla.apply(gorilla.Patch(keras.Model, 'fit', fit, settings=settings))


    def off_autolog():
        def stop_log(self, original, args, kwargs, unlogged_params):
            fit_model = original(self, *args, **kwargs)
            return(fit_model)

        @gorilla.patch(keras.Model)
        def fit(self, *args, **kwargs):
            original = gorilla.get_original_attribute(keras.Model, 'fit')
            unlogged_params = ['self', 'x', 'y', 'callbacks', 'validation_data', 'verbose']
            return stop_log(self, original, args, kwargs, unlogged_params)

        settings = gorilla.Settings(allow_hit=True, store_hit=True)
        gorilla.apply(gorilla.Patch(keras.Model, 'fit', fit, settings=settings))



import json
from Accuinsight.modeler.core.Run.RunInfo.RunInfo import get_current_run
from Accuinsight.modeler.core.LcConst.LcConst import RUN_OBJ_JSON_PATH, RUN_BASE_PATH
from Accuinsight.modeler.core.LcConst.LcConst import ALL_MODEL_PARAMS, SELECTED_METRICS, SELECTED_PARAMS
from Accuinsight.modeler.protos.life_cycle_pb2 import LcParam, LcMetric


# load json data
def _load_current_run():
    run_info_json = get_current_run()
    data_path = run_info_json[RUN_BASE_PATH] + run_info_json[RUN_OBJ_JSON_PATH]

    with open(data_path) as json_file:
        json_data = json.load(json_file)

    return json_data


def _makeLcParam(selected_params, all_model_params):
    grpc_params = []

    for param_key in selected_params:

        param = LcParam(
            key=param_key,
            value=str(all_model_params[param_key])
        )
        grpc_params.append(param)
    return grpc_params


# parse parameters
def parse_run_parameters(json_data):
    selected_params = json_data[SELECTED_PARAMS]
    all_model_params = json_data[ALL_MODEL_PARAMS]

    grpc_params = _makeLcParam(selected_params=selected_params, all_model_params=all_model_params)
    return grpc_params


# parse metrics
def parse_run_metrics(json_data):
    grpc_metrics = []

    # selected_metrics
    selected_metrics = json_data[SELECTED_METRICS]
    dummy_data = {
        'rmse': 9.01244
      },

    metric_keys = selected_metrics.keys()

    result_dict = {'metrics': []}

    count = 0
    for key in metric_keys:
        timestamp = []
        steps = []
        values = []

        values.append(str(selected_metrics[key]))
        timestamp.append(str(count))
        steps.append(str(count))
        count += 1

        result_dict['metrics'].append({'key': key,
                                       'values': values,
                                       'timestamp': timestamp,
                                       'steps': steps})

    metrics = result_dict['metrics']

    for item in metrics:
        metric = LcMetric(
                key=item['key'],
                values=item['values'],
                timestamp=item['timestamp'],
                steps=item['steps']
            )
        grpc_metrics.append(metric)

    return grpc_metrics


def parse_run_result():
    result_dict = {'metrics': '', 'params': ''}
    json_data = _load_current_run()
    grpc_params = parse_run_parameters(json_data=json_data)
    grpc_metrics = parse_run_metrics(json_data=json_data)

    result_dict['metrics'] = grpc_metrics
    result_dict['params'] = grpc_params
    return result_dict




import logging

from Accuinsight.modeler.utils.rest_utils import http_request, verify_rest_response
from Accuinsight.modeler.utils.os_getenv import get_os_env
from Accuinsight.modeler.core.MonitoringConst.deploy import DeployConst


class RestStore:
    def __init__(self, host_creds):
        if host_creds is None:
            raise Exception("host_creds cannot be None")

        self._host_creds = host_creds

    @property
    def host_creds(self):
        return self._host_creds

    @staticmethod
    def set_endpoint_uri(endpoint, mode):
        if mode == "alarm":
            return endpoint + '/monitoring/alarm'
        elif mode == "log_summary":
            return endpoint + '/monitoring/summary'
        else:
            return endpoint + '/monitoring'

    def call_endpoint(self, method, json_body, mode):
        env_value = get_os_env('MONITORING_DEPLOY')
        endpoint = self.set_endpoint_uri('/project/{projectId}/deploy/{deployId}', mode)
        endpoint = endpoint.replace('{projectId}', str(env_value[DeployConst.ENV_PROJECT_ID]))
        endpoint = endpoint.replace('{deployId}', str(env_value[DeployConst.ENV_DEPLOY_ID]))

        try:
            if method == 'GET':
                response = http_request(
                    host_creds=self.host_creds, endpoint=endpoint, method=method)
            else:
                response = http_request(
                    host_creds=self.host_creds, endpoint=endpoint, method=method, data=json_body)

            response = verify_rest_response(response, endpoint).text

        except Exception as e:
            logging.error("Modeler API server connection failed: ", e)
            response = None

        return response



import os
import sys
import json
import logging
import time
import datetime
from collections import OrderedDict

import boto3
from hdfs3 import HDFileSystem
from Accuinsight.modeler.core.LcConst.LcConst import ENV_JUPYTER_HOME_DIR
from Accuinsight.modeler.utils.runs_utils import ProgressPercentage
from Accuinsight.modeler.clients.modeler_api import WorkspaceRestApi
from Accuinsight.modeler.utils.os_getenv import get_os_env, get_current_notebook
from Accuinsight.modeler.core.LcConst import LcConst
from Accuinsight.modeler.entities.alarm import Alarm


class Common(object):
    def __init__(self):
        env_value = get_os_env('ENV')

        self.StorageInfo = None
        self.target_name = None
        self.save_path = None
        self.endpoint = None
        self.thresholds = None
        self.hook_url = None
        self.message = None
        self.feature_name = None
        self.data = None
        self.user_id = None
        self.storage_info_from_set = OrderedDict()
        self.workspace_alarm_api = WorkspaceRestApi(env_value[LcConst.BACK_END_API_URL],
                                                    env_value[LcConst.BACK_END_API_PORT],
                                                    env_value[LcConst.BACK_END_API_URI])
        self.workspace_alarm = Alarm()

        self.notebook_info = get_current_notebook()

    def set_storage(self,
                    access_key=None,  # aws-s3
                    secret_key=None,  # aws-s3
                    region=None,  # aws-s3
                    bucket_name=None,  # aws-s3
                    hdfs_uri=None,  # pipeline-hdfs
                    file_path=None,  # common
                    target=None,  # common
                    save_json=False,  # common - if it is TRUE, saving a storage information as a json file.
                    save_path=None  # common - if save_path is not None, the json file is saved in this path.
                    ):

        if target is None:
            raise ValueError('target을 입력해주시기 바랍니다.')

        ## make a directory for saving json file
        save_prefix = None
        if save_json:
            save_prefix_run = os.path.join(ENV_JUPYTER_HOME_DIR, 'runs')
            if not os.path.isdir(save_prefix_run):
                try:
                    os.mkdir(save_prefix_run)
                except:
                    pass
            save_prefix = os.path.join(save_prefix_run, 'storage-info-json')
            if not os.path.isdir(save_prefix):
                try:
                    os.mkdir(save_prefix)
                except:
                    pass

        ### HDFS
        if hdfs_uri is not None:
            storage_type = 'hdfs'

            split_1 = hdfs_uri.split('//')[1].split('/')
            split_2 = [i.split(':') for i in split_1 if ':' in i][0]

            self.storage_info_from_set['host'] = split_2[0]
            self.storage_info_from_set['port'] = split_2[1]
            self.storage_info_from_set['filePath'] = os.path.join('/', *split_1[1:])
            self.storage_info_from_set['target'] = target

        ### AWS - S3
        elif bucket_name is not None:
            storage_type = 's3'

            self.storage_info_from_set['myAccessKey'] = access_key
            self.storage_info_from_set['mySecretKey'] = secret_key
            self.storage_info_from_set['endpoint'] = 's3.' + region + '.amazonaws.com'
            self.storage_info_from_set['region'] = region
            self.storage_info_from_set['bucketType'] = 's3'
            self.storage_info_from_set['bucketName'] = bucket_name
            self.storage_info_from_set['filePath'] = file_path
            self.storage_info_from_set['fileName'] = file_path.split('/')[-1]
            self.storage_info_from_set['target'] = target

        ### Local
        else:
            storage_type = 'local'
            self.storage_info_from_set['filePath'] = file_path
            self.storage_info_from_set['fileName'] = file_path.split('/')[-1]
            self.storage_info_from_set['target'] = target

            self.save_path = file_path
            self.StorageInfo = self.storage_info_from_set
            self.target_name = target

        if save_json:
            if save_path is None:
                timestamp = str(datetime.datetime.now().date()).replace('-', '')
                dir_list = os.listdir(save_prefix)
                num = len([True for i in dir_list if timestamp in i])
                save_path = timestamp + '-' + str(num + 1).zfill(2) + '-' + storage_type + '-connection-info' + '.json'

            else:  # save_path is not None
                pass

            storage_info_full_path = os.path.join(save_prefix, save_path)
            with open(storage_info_full_path, 'w', encoding='utf-8') as save_file:
                json.dump(self.storage_info_from_set, save_file, indent='\t')

        if not save_json and save_path is not None:
            raise ValueError("'save_json=True'로 설정하시기 바랍니다.")

    def get_file(self, storage_json_file_name=None):
        if storage_json_file_name is not None:

            # meta file 저장하는 폴더 만들면, 이 경로는 수정돼야 함.
            storage_info_json_path = os.path.join(ENV_JUPYTER_HOME_DIR, storage_json_file_name)

            with open(storage_info_json_path) as jsonFile:
                self.StorageInfo = json.load(jsonFile)

        else:  # storage_json_file_name == None, when using set_storage() method.
            self.StorageInfo = self.storage_info_from_set

        ## define the target name
        self.target_name = self.StorageInfo['target']

        ### HDFS
        if 'host' in self.StorageInfo.keys():

            ## path for saving data
            save_dir = os.path.join(ENV_JUPYTER_HOME_DIR, 'data_from_hdfs')
            if os.path.exists(save_dir) == False:
                os.mkdir(save_dir)
            else:
                pass

            split_list = list(filter(None, self.StorageInfo['filePath'].split('/')))

            HOST = self.StorageInfo['host']
            PORT = int(self.StorageInfo['port'])
            FILE_PATH = self.StorageInfo['filePath']
            FILE_DIR = '/'
            for i in split_list[:len(split_list) - 1]:
                FILE_DIR = os.path.join(FILE_DIR, i)
            FILE_NAME = split_list[-1]

            save_file_name = FILE_NAME.split('.')[0] + '_' + str(datetime.datetime.now().date()).replace('-', '') + '.'\
                             + FILE_NAME.split('.')[1]
            self.save_path = os.path.join(save_dir, save_file_name)
            self.endpoint = os.path.join('hdfs://', HOST, self.StorageInfo['port']) + FILE_PATH

            hdfs = HDFileSystem(host=HOST, port=PORT)
            hdfs.ls(FILE_DIR)

            sys.stdout.write('%s %s %s' % ('Downloading file...', FILE_NAME, '\n'))
            time.sleep(1)
            hdfs.get(FILE_PATH, self.save_path)
            logging.info(self.save_path)

        ### AWS-S3
        elif 'bucketName' in self.StorageInfo.keys():
            BUCKET_TYPE = self.StorageInfo['bucketType']
            BUCKET_NAME = self.StorageInfo['bucketName']
            FILE_PATH = self.StorageInfo['filePath']
            FILE_NAME = self.StorageInfo['fileName']
            # FILE_TYPE = self.StorageInfo['fileType']
            # FILE_DELIM = self.StorageInfo['fileDelim']
            ACCESS_KEY = self.StorageInfo['myAccessKey']
            SECRET_KEY = self.StorageInfo['mySecretKey']
            REGION = self.StorageInfo['region']
            URL = self.StorageInfo['endpoint']

            ## path for saving data
            save_dir = os.path.join(ENV_JUPYTER_HOME_DIR, 'data_from_aws')
            if not os.path.exists(save_dir):
                os.mkdir(save_dir)
            else:
                pass

            save_file_name = FILE_NAME.split('.')[0] + '_' + str(datetime.datetime.now().date()).replace('-', '')\
                                  + '.' + FILE_NAME.split('.')[1]
            self.save_path = os.path.join(save_dir, save_file_name)

            # endpoint
            pre_url = 'https://' + BUCKET_NAME + '.' + URL
            self.endpoint = os.path.join(pre_url, FILE_PATH)

            client = boto3.client(BUCKET_TYPE,
                                  aws_access_key_id=ACCESS_KEY,
                                  aws_secret_access_key=SECRET_KEY,
                                  region_name=REGION)

            transfer = boto3.s3.transfer.S3Transfer(client)

            progress = ProgressPercentage(client, BUCKET_NAME, FILE_PATH)

            sys.stdout.write('%s %s %s' % ('Downloading file...', FILE_NAME, '\n'))
            transfer.download_file(BUCKET_NAME, FILE_PATH, self.save_path, callback=progress)
            logging.info(self.save_path)

        ### Local
        else:
            self.save_path = self.StorageInfo['filePath']

        return self.save_path, self.StorageInfo, self.target_name

    def set_data(self, data=None):
        if data is not None:
            self.data = data

    @staticmethod
    def _send_message(metric=None, current_value=None, message=None, thresholds=None, alarm_object=None, alarm_api=None):
        if alarm_object:
            messages = list()

            if thresholds:
                if current_value >= thresholds:
                    messages.append('[모델 학습 완료] ' + metric + '이 설정하신 thresolds: ' + str(thresholds) + '를 초과하였습니다.')

            if message:
                if isinstance(message, str):
                    messages.append(message)
                elif isinstance(message, list):
                    messages.extend(message)
                else:
                    raise ValueError("Wrong message input detected. Available type is 'str' and 'list'")

            if messages:
                for message in messages:
                    alarm_object.message = message
                    alarm_object.type = "Workspace"
                    alarm_api.call_rest_api(alarm_object.get_alarm_param(), "alarm")

    def set_slack(self, hook_url=None):
        if hook_url:
            self.workspace_alarm.notifiers['slack'] = hook_url

    def unset_slack(self):
        if 'slack' in self.workspace_alarm.notifiers.keys():
            del self.workspace_alarm.notifiers['slack']

    def set_mail(self, address=None):
        if address:
            self.workspace_alarm.notifiers['mail'] = address

    def unset_mail(self):
        if 'mail' in self.workspace_alarm.notifiers.keys():
            del self.workspace_alarm.notifiers['mail']

    def send_message(self, message=None, thresholds=None):
        if message and thresholds:
            raise ValueError("'message'와 'thresholds' 두 개의 arguments를 동시에 입력할 수 없습니다.")
        else:
            self.message = message
            self.thresholds = thresholds

    def unset_message(self):
        self.message = None
        self.thresholds = None

    def set_user_id(self, user_id):
        self.user_id = user_id

    def unset_user_id(self):
        self.user_id = None

    def set_features(self, feature_names):
        if str(type(feature_names)) == "<class 'list'>":
            self.feature_name = feature_names
        else:
            try:
                self.feature_name = feature_names.tolist()
            except:
                pass

            try:
                self.feature_name = list(feature_names)
            except AttributeError:
                raise ValueError('입력값으로 list 또는 pandas.core.indexes.base.Index를 사용하시기 바랍니다.')

        return feature_names

    @staticmethod
    def get_current_run_id():
        try:
            with open('/home/work/runs/run_info.json') as json_file:
                return json.load(json_file)['run_name']
        except FileNotFoundError or KeyError:
            return ''

    def set_current_notebook(self):
        self.notebook_info = get_current_notebook()

    def get_current_notebook(self):
        return self.notebook_info



import json
from enum import Enum
from Accuinsight.modeler.core.Run.RunInfo import RunInfo
from Accuinsight.modeler.core.LcConst import LcConst
from Accuinsight.modeler.core.Run.ParseRun import Parse_MLClassifier
from Accuinsight.modeler.core.Run.ParseRun import Parse_DLRegression
from Accuinsight.modeler.core.Run.ParseRun import Parse_DLClassification
from Accuinsight.modeler.core.Run.ParseRun import Parse_MLRegression


class ModelType(Enum):
    DL_REGRESSION = 1
    DL_CLASSIFICATION = 2
    ML_REGRESSION = 3
    ML_CLASSIFICATION = 4


def run_parser(parser_type):
    if parser_type is ModelType.DL_REGRESSION:
        return Parse_DLRegression.parse_run_result()

    if parser_type is ModelType.DL_CLASSIFICATION:
        return Parse_DLClassification.parse_run_result()

    if parser_type is ModelType.ML_REGRESSION:
        return Parse_MLRegression.parse_run_result()

    if parser_type is ModelType.ML_CLASSIFICATION:
        return Parse_MLClassifier.parse_run_result()


def get_parser_type():
    parser_type = None
    run_info_json = RunInfo.get_current_run()
    run_result_path = run_info_json[LcConst.RUN_RESULT_PATH]

    # data_path = run_result_path[LcConst.RUN_MODEL_JSON_PATH]
    visual_json_path = None
    visual_csv_path = None

    if LcConst.RUN_MODEL_VISUAL_JSON_PATH in run_result_path:
        visual_json_path = run_result_path[LcConst.RUN_MODEL_VISUAL_JSON_PATH]

    if LcConst.RUN_MODEL_VISUAL_CSV_PATH in run_result_path:
        visual_csv_path = run_result_path[LcConst.RUN_MODEL_VISUAL_CSV_PATH]

    # determine parser type
    if visual_json_path is None and visual_csv_path is not None:
        parser_type = ModelType.DL_REGRESSION

    if visual_json_path is not None and visual_csv_path is not None:
        parser_type = ModelType.DL_CLASSIFICATION

        visual_json_path = run_result_path[LcConst.RUN_MODEL_VISUAL_JSON_PATH]
        with open(visual_json_path) as json_file:
            visual_json_data = json.load(json_file)

        if LcConst.TRUE_Y in visual_json_data:
            parser_type = ModelType.DL_REGRESSION

    if visual_json_path is None and visual_csv_path is None:
        parser_type = ModelType.ML_REGRESSION

    if visual_json_path is not None and visual_csv_path is None:
        parser_type = ModelType.ML_CLASSIFICATION

    return parser_type





import os
import json
import logging
from collections import OrderedDict
from Accuinsight.modeler.core.Run.RunOjbject import RunObject
from Accuinsight.modeler.core.LcConst import LcConst

# RUN_MODEL_HDF5_PATH, RUN_RESULT_PATH,
# RUN_MODEL_JSON_PATH, RUN_MODEL_VISUAL_CSV_PATH, RUN_MODEL_VISUAL_JSON_PATH,
# RUN_PREFIX_PATH

_runData = []


def set_current_runs(run_name):
    global _runData

    run_dir = get_or_create_run_directory()

    if len(_runData) > 0:
        _runData.clear()
        #raise Exception("{} is already running".format(_runData[0]))

    run_obj = RunObject()
    run_obj[LcConst.RUN_OBJ_NAME] = run_name
    run_obj[LcConst.RUN_BASE_PATH] = run_dir
    run_obj[LcConst.RUN_RESULT_PATH] = {}

    # Initialize: all paths are none.
    run_obj[LcConst.RUN_OBJ_MODEL_JSON_PATH] = None
    run_obj[LcConst.RUN_OBJ_VISUAL_CSV_PATH] = None
    run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH] = None
    run_obj[LcConst.RUN_OBJ_SHAP_JSON_PATH] = None
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH] = None
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH] = None
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH] = None

    _runData.append(run_obj)

    return _runData


def get_current_run():
    with open(LcConst.ENV_JUPYTER_HOME_DIR + LcConst.RUN_ROOT_PATH + 'run_info.json') as json_file:
        return json.load(json_file)


def _get_run_data_value_by_key():
    global _runData
    run_obj = _runData[0]
    run_meta = OrderedDict()

    for key in run_obj.__dict__.keys():
        run_meta[key] = run_obj[key]

    return run_meta


def clear_runs(start_time, end_time, delta_time, json_path=None):
    global _runData
    run_meta = _get_run_data_value_by_key()
    run_meta[LcConst.START_TIME] = start_time
    run_meta[LcConst.END_TIME] = end_time
    run_meta[LcConst.DELTA_TIME] = delta_time

    save_result_path = run_meta[LcConst.RUN_RESULT_PATH] = {}
    save_result_path[LcConst.RUN_BASE_PATH] = run_meta[LcConst.RUN_BASE_PATH]
    save_result_path[LcConst.RUN_PREFIX_PATH] = run_meta[LcConst.RUN_PREFIX_PATH]
    save_result_path[LcConst.RUN_MODEL_JSON_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                            save_result_path[LcConst.RUN_PREFIX_PATH],
                                                            run_meta[LcConst.RUN_OBJ_MODEL_JSON_PATH]
                                                            ])

    if run_meta[LcConst.RUN_OBJ_VISUAL_CSV_PATH] is not None:
        save_result_path[LcConst.RUN_MODEL_VISUAL_CSV_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                      save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                      run_meta[LcConst.RUN_OBJ_VISUAL_CSV_PATH]
                                                                      ])

    if run_meta[LcConst.RUN_OBJ_VISUAL_JSON_PATH] is not None:
        save_result_path[LcConst.RUN_MODEL_VISUAL_JSON_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                       save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                       run_meta[LcConst.RUN_OBJ_VISUAL_JSON_PATH]
                                                                       ])
    if run_meta[LcConst.RUN_OBJ_SHAP_JSON_PATH] is not None:
        save_result_path[LcConst.RUN_MODEL_SHAP_JSON_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                       save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                       run_meta[LcConst.RUN_OBJ_SHAP_JSON_PATH]
                                                                       ])
    if run_meta[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH ] is not None:
        save_result_path[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                    save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                    run_meta[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH]
                                                                    ])
    if run_meta[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH] is not None:
        save_result_path[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                    save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                    run_meta[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH]
                                                                    ])
    if run_meta[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH] is not None:
        save_result_path[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH] = ''.join([save_result_path[LcConst.RUN_BASE_PATH],
                                                                    save_result_path[LcConst.RUN_PREFIX_PATH],
                                                                    run_meta[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH]
                                                                    ])

    save_path = run_meta[LcConst.RUN_BASE_PATH] + 'run_info.json'
    with open(save_path, 'w', encoding='utf-8') as save_file:
        json.dump(run_meta, save_file, indent="\t")

    _runData.pop()


def _set_result_full_path(dict_path):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_BASE_PATH] = get_or_create_run_directory()

    run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_PREFIX_PATH] = dict_path[LcConst.RUN_PREFIX_PATH]

    run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_MODEL_JSON_PATH] = dict_path[LcConst.RUN_MODEL_JSON_PATH]

    if LcConst.RUN_MODEL_VISUAL_CSV_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_MODEL_VISUAL_CSV_PATH] = dict_path[LcConst.RUN_MODEL_VISUAL_CSV_PATH]

    if LcConst.RUN_MODEL_VISUAL_JSON_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_MODEL_VISUAL_JSON_PATH] = dict_path[LcConst.RUN_MODEL_VISUAL_JSON_PATH]
        
    if LcConst.RUN_MODEL_SHAP_JSON_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_MODEL_SHAP_JSON_PATH] = dict_path[LcConst.RUN_MODEL_SHAP_JSON_PATH]

    if LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH] = dict_path[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH]

    if LcConst.RUN_OBJ_BEST_MODEL_H5_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_OBJ_BEST_MODEL_H5_PATH] = dict_path[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH]

    if LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH in dict_path:
        run_obj[LcConst.RUN_RESULT_PATH][LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH] = dict_path[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH]


def _set_result_path(dict_path):
    set_model_json_path(dict_path['model_json'])

    if 'visual_json' in dict_path:
        set_visual_json_path(dict_path['visual_json'])


def set_run_name(model_type, run_id):
    global _runData

    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_NAME] = "{}-{}".format(model_type, run_id.replace('-', ''))


def set_prefix_path(prefix_path):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_PREFIX_PATH] = prefix_path


def set_model_json_path(json_path):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_MODEL_JSON_PATH] = json_path


def set_visual_csv_path(csv_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_VISUAL_CSV_PATH] = csv_path


def set_visual_json_path(json_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH] = json_path


def set_shap_json_path(shap_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_SHAP_JSON_PATH] = shap_path


def set_best_model_h5_path(hdf5_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH] = 'runs/' + hdf5_path


def set_best_model_json_path(json_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_JSON_PATH] = 'runs/' + json_path


def set_best_model_joblib_path(joblib_path=None):
    global _runData
    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH] = 'runs/' + joblib_path


def get_or_create_run_directory():
    run_dir = LcConst.ENV_JUPYTER_HOME_DIR + LcConst.RUN_ROOT_PATH

    if not os.path.isdir(run_dir):
        try:
            os.mkdir(run_dir)
        except:
            logging.info("Run directory is not created")

    return run_dir


def set_git_meta(fileinfo):
    # fileinfo (dict_keys(['filename', 'digest', 'repo', 'commit', 'is_dirty', 'run_path']))
    global _runData

    run_obj = _runData[0]
    run_obj[LcConst.SOURCE_FILE_GIT_META] = fileinfo


def set_model_file_path(model_file_path):
    global _runData

    run_obj = _runData[0]
    run_obj[LcConst.RUN_OBJ_MODEL_FILE_PATH] = model_file_path


def set_python_dependencies(py_depenpency):
    global _runData

    run_obj = _runData[0]
    run_obj[LcConst.PYTHON_DEPENDENCY] = py_depenpency


def dict_to_list(dict_data):
    # "dependency": {
    #     "numpy": "1.18.2",
    #     "scikit-learn": "0.22.2.post1",
    #     "xgboost": "1.1.0rc1",
    #     "ModelLifeCycle": "0.1.0"
    # }

    # [
    #     'Numpy==1.18.1',
    #     'Pandas==1.0.1',
    #     'LightGBM==2.3.2'
    # ]
    return ["{}=={}".format(key, dict_data[key] or "<unknown>") for key in dict_data.keys()]


def print_run_info():
    run_obj = get_current_run()

    # DL & Classification
    if 'keras' in run_obj[LcConst.RUN_OBJ_NAME]:
        print(f'\nThe outputs of run: \n'
              f'best_model_save_path: {run_obj[LcConst.RUN_OBJ_BEST_MODEL_H5_PATH]} \n'
              f'model_info_json_path: {run_obj[LcConst.RUN_OBJ_MODEL_JSON_PATH]} \n'
              f'model_history_csv_path: {run_obj[LcConst.RUN_OBJ_VISUAL_CSV_PATH]} \n'
              f'visual_json_path: {run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH]} \n')

    # ML & Classification
    elif run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH] is not None:
        print(f'\nThe outputs of run: \n'
              f'best_model_save_path: {run_obj[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH]} \n'
              f'model_info_json_path: {run_obj[LcConst.RUN_OBJ_MODEL_JSON_PATH]} \n'
              f'visual_json_path: {run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH]} \n')

    # ML & Regression
    elif run_obj[LcConst.RUN_OBJ_VISUAL_JSON_PATH] is None:
        print(f'\nThe outputs of run: \n'
              f'best_model_save_path: {run_obj[LcConst.RUN_OBJ_BEST_MODEL_JOBLIB_PATH]} \n'
              f'model_info_json_path: {run_obj[LcConst.RUN_OBJ_MODEL_JSON_PATH]}')



from setuptools import setup, find_packages
from pathlib import Path

classifiers = """
Development Status :: 1 - Planning
Intended Audience :: Science/Research
Natural Language :: English
Operating System :: OS Independent
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.7
Topic :: Utilities
Topic :: Scientific/Engineering
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Software Development :: Libraries :: Python Modules
License :: OSI Approved :: MIT License
"""

try:
    from Accuinsight import __about__

    about = __about__.__dict__
except ImportError:
    about = dict()
    exec(open("modeler/__about__.py").read(), about)

setup(
    name='Accuinsight',
    version=about['__version__'],
    url=about['__url__'],
    author=about['__author__'],
    description='Model life cycle and monitoring library in Accuinsight+',
    long_description=Path("README.rst").read_text(encoding="utf-8"),
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=Path("requirements.txt").read_text().splitlines(),
    setup_requires=['pytest-runner'],
    tests_require=["mock>=0.8, <3.0", "pytest==4.3.0"],
    classifiers=list(filter(None, classifiers.split("\n"))),
    zip_safe=False
)
